/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.io.IOException;

import jgromacs.data.Alignment;
import jgromacs.data.IndexSet;
import jgromacs.data.IndexSetList;
import jgromacs.data.Sequence;
import jgromacs.db.ResidueType;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class AlignmentTest extends TestCase{


	@Test
	public void testGetMaxSequenceLength() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			assertEquals(105, a.getMaxSequenceLength());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMinSequenceLength() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			assertEquals(91, a.getMinSequenceLength());
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testRemoveColumn() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			Alignment a2 = (Alignment)a.clone();
			
			a.removeColumn(9);
			
			for (int i = 0; i < a.getNumberOfSequences(); i++) {
				Sequence seq = a.getSequence(i);
				Sequence seq2 = a2.getSequence(i);
				assertEquals(seq.getPosition(8), seq2.getPosition(8));
				assertEquals(seq.getPosition(9), seq2.getPosition(10));
			}
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testIsMatchColumn() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			
			assertTrue(!a.isMatchColumn(1));
			assertTrue(a.isMatchColumn(5));
			assertTrue(a.isMatchColumn(10));
			assertTrue(!a.isMatchColumn(19));
			assertTrue(a.isMatchColumn(23));
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetCollapsedAlignment() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			
			Alignment ca = a.getCollapsedAlignment();
			
			// 1st column in collapsed alignment:
			assertEquals(new ResidueType("A"), ca.getSequence(0).getPosition(0).getResidueType());
			assertEquals(new ResidueType("P"), ca.getSequence(1).getPosition(0).getResidueType());
			assertEquals(new ResidueType("G"), ca.getSequence(2).getPosition(0).getResidueType());
			assertEquals(new ResidueType("P"), ca.getSequence(3).getPosition(0).getResidueType());
			assertEquals(new ResidueType("G"), ca.getSequence(4).getPosition(0).getResidueType());
			
			// 15th column in collapsed alignment:
			assertEquals(new ResidueType("S"), ca.getSequence(0).getPosition(14).getResidueType());
			assertEquals(new ResidueType("D"), ca.getSequence(1).getPosition(14).getResidueType());
			assertEquals(new ResidueType("E"), ca.getSequence(2).getPosition(14).getResidueType());
			assertEquals(new ResidueType("S"), ca.getSequence(3).getPosition(14).getResidueType());
			assertEquals(new ResidueType("F"), ca.getSequence(4).getPosition(14).getResidueType());
			
			// 31th column in collapsed alignment:
			assertEquals(new ResidueType("I"), ca.getSequence(0).getPosition(30).getResidueType());
			assertEquals(new ResidueType("V"), ca.getSequence(1).getPosition(30).getResidueType());
			assertEquals(new ResidueType("V"), ca.getSequence(2).getPosition(30).getResidueType());
			assertEquals(new ResidueType("V"), ca.getSequence(3).getPosition(30).getResidueType());
			assertEquals(new ResidueType("I"), ca.getSequence(4).getPosition(30).getResidueType());
			
					
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMatchPositionIndices() {
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			
			IndexSet iset = a.getMatchPositionIndices(2);
			
			assertTrue(!iset.isIndexIn(2));
			assertTrue(iset.isIndexIn(5));
			assertTrue(iset.isIndexIn(12));
			assertTrue(!iset.isIndexIn(21));
			assertTrue(!iset.isIndexIn(59));
			assertTrue(iset.isIndexIn(60));
			
			IndexSet iset2 = a.getMatchPositionIndices(4);
			assertTrue(iset2.isIndexIn(2));
			assertTrue(!iset2.isIndexIn(16));
		
			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMatchPositionIndexSets() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			
			IndexSetList isetlist = a.getMatchPositionIndexSets();
			
			assertEquals(5, isetlist.getNumberOfIndexSets());
			
			IndexSet iset = isetlist.getIndexSet(2); // 3rd index returned
			IndexSet iset2 = isetlist.getIndexSet(4); // 5th index returned
			
			assertTrue(!iset.isIndexIn(2));
			assertTrue(iset.isIndexIn(5));
			assertTrue(iset.isIndexIn(12));
			assertTrue(!iset.isIndexIn(21));
			assertTrue(!iset.isIndexIn(59));
			assertTrue(iset.isIndexIn(60));
			
			assertTrue(iset2.isIndexIn(2));
			assertTrue(!iset2.isIndexIn(16));			
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetMostFrequentResidueType() {
		
		try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
			
			assertEquals(new ResidueType("L"), a.getMostFrequentResidueType(7));	
			assertEquals(new ResidueType("K"), a.getMostFrequentResidueType(62));		
			assertEquals(new ResidueType("N"), a.getMostFrequentResidueType(71));		
			assertEquals(new ResidueType("D"), a.getMostFrequentResidueType(81)); 
			assertEquals(new ResidueType(), a.getMostFrequentResidueType(0));	// Gap is most frequent
				
			} catch (IOException e) {
				fail(e.toString());
			}
	}

	@Test
	public void testGetConsensusSequence() {
	try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
		
			Sequence seq = a.getConsensusSequence();
				
			// Checking a few positions consensus sequence:
			assertEquals(new ResidueType("M"), seq.getPosition(3).getResidueType()); 
			assertEquals(new ResidueType("I"), seq.getPosition(24).getResidueType()); 
			assertEquals(new ResidueType("P"), seq.getPosition(54).getResidueType()); 
			assertEquals(new ResidueType("K"), seq.getPosition(62).getResidueType());
			assertEquals(new ResidueType(), seq.getPosition(95).getResidueType()); // Gap
			assertEquals(new ResidueType("L"), seq.getPosition(99).getResidueType());
			assertEquals(new ResidueType(), seq.getPosition(108).getResidueType()); // Gap
			
			} catch (IOException e) {
				fail(e.toString());
			}
	}
	
	@Test
	public void testCloneEquals() {
	try {
			
			Alignment a = IOData.readAlignmentFromFASTA("dataset//example_alignment.fasta");
		
			Alignment ca = (Alignment)a.clone();
			
			assertTrue(ca.equals(a));
			assertTrue(ca!=a);
			
			a.removeColumn(0);
			assertTrue(!ca.equals(a));
			
	} catch (IOException e) {
		fail(e.toString());
	}	
				
	}

}
