/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import jama.Matrix;

import java.io.IOException;

import jgromacs.analysis.Angles;
import jgromacs.data.Angle;
import jgromacs.data.Point3D;
import jgromacs.data.Structure;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class AnglesTest extends TestCase {

	@Test
	public void testGetAngleBetweenVectors() {

		Point3D v1 = new Point3D(2,3,4);
		Point3D v2 = new Point3D(1,-2,3);
	
		assertEquals(66.6, Angles.getAngleBetweenVectors(v1, v2).getInDegrees(),0.05);
	}

	@Test
	public void testGetAngleBetweenPlanes() {
		
		Point3D A = new Point3D(1,2,3);
		Point3D B = new Point3D(1,1,1);
		Point3D C = new Point3D(2,-1,3);
		Point3D D = new Point3D(3,2,1);
		
		Angle result = Angles.getAngleBetweenPlanes(A, B, C, D);

		assertEquals(77.9, result.getInDegrees(),0.05);
	}

	@Test
	public void testGetDihedralPhi() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			// Control values are calculated with the Whatif server:
			
			Angle result = Angles.getDihedralPhi(s, 20);		
			assertEquals(-90.9, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralPhi(s, 6);		
			assertEquals(-127.6, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralPhi(s, 23);		
			assertEquals(78.6, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralPhi(s, 67);		
			assertEquals(-155.8, result.getInDegrees(), 0.05);
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetDihedralPsi() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			// Control values are calculated with the Whatif server:
			
			Angle result = Angles.getDihedralPsi(s, 20);		
			assertEquals(12.0, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralPsi(s, 6);		
			assertEquals(129.8, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralPsi(s, 23);		
			assertEquals(13.3, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralPsi(s, 67);		
			assertEquals(144.0, result.getInDegrees(), 0.05);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDihedralOmega() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			// Control values are calculated with the Whatif server:
			
			Angle result = Angles.getDihedralOmega(s, 20);		
			assertEquals(178.8, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralOmega(s, 6);		
			assertEquals(177.3, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralOmega(s, 23);		
			assertEquals(174.4, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralOmega(s, 67);		
			assertEquals(176.5, result.getInDegrees(), 0.05);
			
			} catch (IOException e) {
				fail(e.toString());
			}		


	}

	@Test
	public void testGetDihedralChi1() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			// Control values are calculated with the Whatif server:
			
			Angle result = Angles.getDihedralChi1(s, 19);		
			assertEquals(75.7, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi1(s, 16);		
			assertEquals(176.9, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi1(s, 27);		
			assertEquals(-169.1, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi1(s, 23);		
			assertNull(result);
			
			} catch (IOException e) {
				fail(e.toString());
			}		


	}

	@Test
	public void testGetDihedralChi2() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			// Control values are calculated with the Whatif server:
			
			Angle result = Angles.getDihedralChi2(s, 5);		
			assertEquals(104.0, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi2(s, 21);		
			assertEquals(-80.0, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi2(s, 19);		
			assertEquals(-5.4, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi2(s, 92);		
			assertNull(result);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDihedralChi3() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			// Control values are calculated with the Whatif server:
			
			Angle result = Angles.getDihedralChi3(s, 86);		
			assertEquals(172.4, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi3(s, 77);		
			assertEquals(8.9, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi3(s, 9);		
			assertEquals(-174.8, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi3(s, 8);		
			assertNull(result);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDihedralChi4() {
		try {
			
			Structure s = IOData.readStructureFromPDB("dataset//example_single.pdb");
		
			// Control values are calculated with the Whatif server:
			
			Angle result = Angles.getDihedralChi4(s, 9);		
			assertEquals(171.4, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi4(s, 15);		
			assertEquals(-83.9, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi4(s, 64);		
			assertEquals(170.7, result.getInDegrees(), 0.05);
			
			result = Angles.getDihedralChi4(s, 60);		
			assertNull(result);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	
	@Test
	public void testGetRamachandranPlot() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_protein.gro");
			
			Matrix result = Angles.getRamachandranPlot(s);
			
			assertEquals(91, result.getRowDimension());
			
			assertEquals(-113.1, result.get(0, 0),0.05);
			assertEquals(154.3, result.get(0, 1),0.05);
			
			assertEquals(121.8, result.get(20, 0),0.05);
			assertEquals(168.6, result.get(20, 1),0.05);
			
			assertEquals(-100.6, result.get(90, 0),0.05);
			assertEquals(149.2, result.get(90, 1),0.05);
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

}
