/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import jama.Matrix;

import java.util.ArrayList;

import jgromacs.data.Point3D;
import jgromacs.data.PointList;

import junit.framework.TestCase;

import org.junit.Test;

public class PointListTest extends TestCase{

	@Test
	public void testGetSubList() {

		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(2,2,2));
		pl.addPoint(new Point3D(3,3,3));
		pl.addPoint(new Point3D(4,4,4));
		pl.addPoint(new Point3D(5,5,5));
		pl.addPoint(new Point3D(6,6,6));
		pl.addPoint(new Point3D(7,7,7));
		
		ArrayList<Integer> indices = new ArrayList<Integer>();
		indices.add(1);
		indices.add(3);
		indices.add(6);
			
		PointList pl2 = pl.getSubList(indices); //  Getting a sublist
	
		PointList control = new PointList();
		control.addPoint(new Point3D(2,2,2));
		control.addPoint(new Point3D(4,4,4));
		control.addPoint(new Point3D(7,7,7));
	
		assertEquals(control, pl2);
	}

	@Test
	public void testGetAsMatrix() {
		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(2,2,2));
		
		Matrix M = pl.getAsMatrix(); // Getting as a matrix
		
		assertEquals(1, M.get(0, 0),0.0000001);
		assertEquals(1, M.get(1, 0),0.0000001);
		assertEquals(1, M.get(2, 0),0.0000001);
		assertEquals(2, M.get(0, 1),0.0000001);
		assertEquals(2, M.get(1, 1),0.0000001);
		assertEquals(2, M.get(2, 1),0.0000001);
		
	}

	@Test
	public void testLoadFromMatrix() {
	
		PointList pl = new PointList();
		
		Matrix M = new Matrix(3,4);
		M.set(0, 0, 1);
		M.set(1, 0, 1);
		M.set(2, 0, 1);
		M.set(0, 1, 2);
		M.set(1, 1, 2);
		M.set(2, 1, 2);
		M.set(0, 2, 3);
		M.set(1, 2, 3);
		M.set(2, 2, 3);
		M.set(0, 3, 4);
		M.set(1, 3, 4);
		M.set(2, 3, 4);
		
		pl.loadFromMatrix(M); // Loading data from matrix
		
		assertEquals(new Point3D(1,1,1), pl.getPoint(0));
		assertEquals(new Point3D(3,3,3), pl.getPoint(2));
		assertEquals(new Point3D(4,4,4), pl.getPoint(3));
	}

	@Test
	public void testGetCentroid() {
		
		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(2,2,2));
		pl.addPoint(new Point3D(3,3,3));
		pl.addPoint(new Point3D(4,4,4));
		pl.addPoint(new Point3D(5,5,5));
		pl.addPoint(new Point3D(6,6,6));
		pl.addPoint(new Point3D(7,7,7));
		
		assertEquals(new Point3D(4,4,4), pl.getCentroid());
	}
	
	@Test
	public void testCenterPoints() {

		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(2,2,2));
		pl.addPoint(new Point3D(3,3,3));
		pl.addPoint(new Point3D(4,4,4));
		pl.addPoint(new Point3D(5,5,5));
		pl.addPoint(new Point3D(6,6,6));
		pl.addPoint(new Point3D(7,7,7));
		
		assertEquals(new Point3D(4,4,4), pl.getCentroid());
		
		pl.centerPoints(); // Centering points
		
		assertEquals(new Point3D(0,0,0), pl.getCentroid());
	}


	@Test
	public void testRotate() {
		
		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(1,2,3));
			
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, -1);
		rot.set(0, 2, 0);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 0);
		rot.set(2, 2, 1);
		
		pl.rotate(rot); // +90 degrees rotation about the z-axis
		
		PointList control = new PointList();
		control.addPoint(new Point3D(-1,1,1));
		control.addPoint(new Point3D(-2,1,3));
		
		assertEquals(control, pl);
	}

	@Test
	public void testTranslate() {
		
		PointList pl = new PointList();
		pl.addPoint(new Point3D(1,1,1));
		pl.addPoint(new Point3D(1,2,3));
	
		Point3D vector = new Point3D(4,4,-4);
	
		pl.translate(vector); // Translating points
		
		PointList control = new PointList();
		control.addPoint(new Point3D(5,5,-3));
		control.addPoint(new Point3D(5,6,-1));
		
		assertEquals(control, pl);
	}

	@Test
	public void testCloneEquals() {
	
		PointList x = new PointList();
		x.addPoint(new Point3D(1,1,1));
		x.addPoint(new Point3D(1,2,3));
		
		PointList cx = (PointList)x.clone();
		
		assertTrue(x.equals(cx));
		assertTrue(x!=cx);
		
		x.removePoint(0);
		assertTrue(!x.equals(cx));
	}
	
}
