/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import jama.Matrix;

import java.io.IOException;
import java.util.ArrayList;

import jgromacs.analysis.Distances;
import jgromacs.analysis.Similarity;
import jgromacs.data.FrameIndexSet;
import jgromacs.data.IndexSet;
import jgromacs.data.Point3D;
import jgromacs.data.PointList;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import jgromacs.io.IOData;
import junit.framework.TestCase;

import org.junit.Test;

public class SimilarityTest extends TestCase{

	@Test
	public void testGetRMSDPointListPointList() {
		try {
				
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			double result = Similarity.getRMSD(s1.getAllAtomCoordinates(), s2.getAllAtomCoordinates());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0395597, result, 0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetRMSDStructureStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			double result = Similarity.getRMSD(s1, s2);
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0395597, result, 0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		


	}

	@Test
	public void testGetRMSDStructureIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			double result = Similarity.getRMSD(s1, iset, s2, iset);
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0395597, result, 0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		


	}

	@Test
	public void testGetRMSDiProfilePointListPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			ArrayList<Double> result = Similarity.getRMSDiProfile(s1.getAllAtomCoordinates(), s2.getAllAtomCoordinates());
			
			assertEquals(4, result.size());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0064, result.get(0), 0.0001);
			assertEquals(0.0348, result.get(1), 0.0001);
			assertEquals(0.0615, result.get(2), 0.0001);
			assertEquals(0.0347, result.get(3), 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetRMSDiProfileStructureStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			ArrayList<Double> result = Similarity.getRMSDiProfile(s1, s2);
			
			assertEquals(4, result.size());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0064, result.get(0), 0.0001);
			assertEquals(0.0348, result.get(1), 0.0001);
			assertEquals(0.0615, result.get(2), 0.0001);
			assertEquals(0.0347, result.get(3), 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetRMSDiProfileStructureIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
				
			ArrayList<Double> result = Similarity.getRMSDiProfile(s1, iset, s2, iset);
				
			assertEquals(4, result.size());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0064, result.get(0), 0.0001);
			assertEquals(0.0348, result.get(1), 0.0001);
			assertEquals(0.0615, result.get(2), 0.0001);
			assertEquals(0.0347, result.get(3), 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetRMSDnoSuperpositionPointListPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			double result = Similarity.getRMSDnoSuperposition(s1.getAllAtomCoordinates(), s2.getAllAtomCoordinates());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0860523, result, 0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetRMSDnoSuperpositionStructureStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			double result = Similarity.getRMSDnoSuperposition(s1, s2);
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0860523, result, 0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetRMSDnoSuperpositionStructureIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			double result = Similarity.getRMSDnoSuperposition(s1, iset, s2, iset);
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0860523, result, 0.00001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		


	}

	@Test
	public void testGetRMSDiProfileNoSuperpositionPointListPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			ArrayList<Double> result = Similarity.getRMSDiProfileNoSuperposition(s1.getAllAtomCoordinates(), s2.getAllAtomCoordinates());
			
			assertEquals(4, result.size());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0669, result.get(0), 0.0001);
			assertEquals(0.0801, result.get(1), 0.0001);
			assertEquals(0.1185, result.get(2), 0.0001);
			assertEquals(0.0681, result.get(3), 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetRMSDiProfileNoSuperpositionStructureStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			s1 = s1.getSubStructure(iset);
			s2 = s2.getSubStructure(iset);
			
			ArrayList<Double> result = Similarity.getRMSDiProfileNoSuperposition(s1, s2);
			
			assertEquals(4, result.size());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0669, result.get(0), 0.0001);
			assertEquals(0.0801, result.get(1), 0.0001);
			assertEquals(0.1185, result.get(2), 0.0001);
			assertEquals(0.0681, result.get(3), 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		


	}

	@Test
	public void testGetRMSDiProfileNoSuperpositionStructureIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			iset.addIndex(40);
			
			Structure s1 = t.getFrameAsStructure(9);
			Structure s2 = t.getFrameAsStructure(17);
			
			ArrayList<Double> result = Similarity.getRMSDiProfileNoSuperposition(s1, iset, s2, iset);
			
			assertEquals(4, result.size());
			
			// Control value is calculated with the Gromacs command g_confrms
			assertEquals(0.0669, result.get(0), 0.0001);
			assertEquals(0.0801, result.get(1), 0.0001);
			assertEquals(0.1185, result.get(2), 0.0001);
			assertEquals(0.0681, result.get(3), 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDMatrixMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix M1 = Distances.getDistanceMatrix(frame1);
			Matrix M2 = Distances.getDistanceMatrix(frame2);
			double result = Similarity.getDRMSD(M1,M2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control = 2*dd_01*dd_01+2*dd_02*dd_02+2*dd_12*dd_12;
			control = Math.sqrt(control/9);
			
			assertEquals(control, result, 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDPointListPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();

			double result = Similarity.getDRMSD(frame1,frame2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control = 2*dd_01*dd_01+2*dd_02*dd_02+2*dd_12*dd_12;
			control = Math.sqrt(control/9);
			
			assertEquals(control, result, 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDStructureStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();

			double result = Similarity.getDRMSD(frameS1,frameS2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control = 2*dd_01*dd_01+2*dd_02*dd_02+2*dd_12*dd_12;
			control = Math.sqrt(control/9);
			
			assertEquals(control, result, 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDStructureIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);
			
			Structure s1 = (Structure)frameS1.clone();
			Structure s2 = (Structure)frameS2.clone();
			
			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);	
			
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();

			double result = Similarity.getDRMSD(s1,iset,s2,iset);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control = 2*dd_01*dd_01+2*dd_02*dd_02+2*dd_12*dd_12;
			control = Math.sqrt(control/9);
			
			assertEquals(control, result, 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDiProfileMatrixMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix M1 = Distances.getDistanceMatrix(frame1);
			Matrix M2 = Distances.getDistanceMatrix(frame2);
			ArrayList<Double> result = Similarity.getDRMSDiProfile(M1, M2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control0 = Math.sqrt((dd_01*dd_01+dd_02*dd_02)/3);
			double control1 = Math.sqrt((dd_01*dd_01+dd_12*dd_12)/3);
			double control2 = Math.sqrt((dd_02*dd_02+dd_12*dd_12)/3);
			
			assertEquals(control0, result.get(0), 0.0001);
			assertEquals(control1, result.get(1), 0.0001);
			assertEquals(control2, result.get(2), 0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDiProfilePointListPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			ArrayList<Double> result = Similarity.getDRMSDiProfile(frame1, frame2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control0 = Math.sqrt((dd_01*dd_01+dd_02*dd_02)/3);
			double control1 = Math.sqrt((dd_01*dd_01+dd_12*dd_12)/3);
			double control2 = Math.sqrt((dd_02*dd_02+dd_12*dd_12)/3);
			
			assertEquals(control0, result.get(0), 0.0001);
			assertEquals(control1, result.get(1), 0.0001);
			assertEquals(control2, result.get(2), 0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDiProfileStructureStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			ArrayList<Double> result = Similarity.getDRMSDiProfile(frameS1, frameS2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control0 = Math.sqrt((dd_01*dd_01+dd_02*dd_02)/3);
			double control1 = Math.sqrt((dd_01*dd_01+dd_12*dd_12)/3);
			double control2 = Math.sqrt((dd_02*dd_02+dd_12*dd_12)/3);
			
			assertEquals(control0, result.get(0), 0.0001);
			assertEquals(control1, result.get(1), 0.0001);
			assertEquals(control2, result.get(2), 0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDRMSDiProfileStructureIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			Structure s1 = (Structure)frameS1.clone();
			Structure s2 = (Structure)frameS2.clone();
			
			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			ArrayList<Double> result = Similarity.getDRMSDiProfile(s1, iset, s2, iset);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control0 = Math.sqrt((dd_01*dd_01+dd_02*dd_02)/3);
			double control1 = Math.sqrt((dd_01*dd_01+dd_12*dd_12)/3);
			double control2 = Math.sqrt((dd_02*dd_02+dd_12*dd_12)/3);
			
			assertEquals(control0, result.get(0), 0.0001);
			assertEquals(control1, result.get(1), 0.0001);
			assertEquals(control2, result.get(2), 0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetWDRMSDPointListPointListMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix W = Matrix.random(3, 3);
			
			double result = Similarity.getWDRMSD(frame1,frame2,W);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control = W.get(0, 1)*dd_01*dd_01+W.get(1, 0)*dd_01*dd_01+
							+W.get(0, 2)*dd_02*dd_02+W.get(2, 0)*dd_02*dd_02+
							+W.get(1, 2)*dd_12*dd_12+W.get(2, 1)*dd_12*dd_12;
			
			double sumW = 0;
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j < 3; j++) {
					sumW+=W.get(i, j);
				}
			}
			
			control = Math.sqrt(control/sumW);
			
			assertEquals(control, result, 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetWDRMSDStructureStructureMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix W = Matrix.random(3, 3);
			
			double result = Similarity.getWDRMSD(frameS1,frameS2,W);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control = W.get(0, 1)*dd_01*dd_01+W.get(1, 0)*dd_01*dd_01+
							+W.get(0, 2)*dd_02*dd_02+W.get(2, 0)*dd_02*dd_02+
							+W.get(1, 2)*dd_12*dd_12+W.get(2, 1)*dd_12*dd_12;
			
			double sumW = 0;
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j < 3; j++) {
					sumW+=W.get(i, j);
				}
			}
			
			control = Math.sqrt(control/sumW);
			
			assertEquals(control, result, 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetWDRMSDStructureIndexSetStructureIndexSetMatrix() {
	try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			Structure s1 = (Structure)frameS1.clone();
			Structure s2 = (Structure)frameS2.clone();
			
			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix W = Matrix.random(3, 3);
			
			double result = Similarity.getWDRMSD(s1, iset, s2, iset, W);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control = W.get(0, 1)*dd_01*dd_01+W.get(1, 0)*dd_01*dd_01+
							+W.get(0, 2)*dd_02*dd_02+W.get(2, 0)*dd_02*dd_02+
							+W.get(1, 2)*dd_12*dd_12+W.get(2, 1)*dd_12*dd_12;
			
			double sumW = 0;
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j < 3; j++) {
					sumW+=W.get(i, j);
				}
			}
			
			control = Math.sqrt(control/sumW);
			
			assertEquals(control, result, 0.0001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetWDRMSDiProfilePointListPointListMatrix() {
	try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix W = Matrix.random(3, 3);
			
			ArrayList<Double> result = Similarity.getWDRMSDiProfile(frame1, frame2, W);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control0 = Math.sqrt((W.get(0, 1)*dd_01*dd_01+W.get(0, 2)*dd_02*dd_02)/(W.get(0, 0)+W.get(0, 1)+W.get(0, 2)));
			double control1 = Math.sqrt((W.get(1, 0)*dd_01*dd_01+W.get(1, 2)*dd_12*dd_12)/(W.get(1, 0)+W.get(1, 1)+W.get(1, 2)));
			double control2 = Math.sqrt((W.get(2, 0)*dd_02*dd_02+W.get(2, 1)*dd_12*dd_12)/(W.get(2, 0)+W.get(2, 1)+W.get(2, 2)));
			
			assertEquals(control0, result.get(0), 0.0001);
			assertEquals(control1, result.get(1), 0.0001);
			assertEquals(control2, result.get(2), 0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetWDRMSDiProfileStructureStructureMatrix() {
	try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix W = Matrix.random(3, 3);
			
			ArrayList<Double> result = Similarity.getWDRMSDiProfile(frameS1, frameS2, W);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control0 = Math.sqrt((W.get(0, 1)*dd_01*dd_01+W.get(0, 2)*dd_02*dd_02)/(W.get(0, 0)+W.get(0, 1)+W.get(0, 2)));
			double control1 = Math.sqrt((W.get(1, 0)*dd_01*dd_01+W.get(1, 2)*dd_12*dd_12)/(W.get(1, 0)+W.get(1, 1)+W.get(1, 2)));
			double control2 = Math.sqrt((W.get(2, 0)*dd_02*dd_02+W.get(2, 1)*dd_12*dd_12)/(W.get(2, 0)+W.get(2, 1)+W.get(2, 2)));
			
			assertEquals(control0, result.get(0), 0.0001);
			assertEquals(control1, result.get(1), 0.0001);
			assertEquals(control2, result.get(2), 0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetWDRMSDiProfileStructureIndexSetStructureIndexSetMatrix() {
	try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			Structure s1 = (Structure)frameS1.clone();
			Structure s2 = (Structure)frameS2.clone();
			
			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix W = Matrix.random(3, 3);
			
			ArrayList<Double> result = Similarity.getWDRMSDiProfile(s1, iset, s2, iset, W);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			double control0 = Math.sqrt((W.get(0, 1)*dd_01*dd_01+W.get(0, 2)*dd_02*dd_02)/(W.get(0, 0)+W.get(0, 1)+W.get(0, 2)));
			double control1 = Math.sqrt((W.get(1, 0)*dd_01*dd_01+W.get(1, 2)*dd_12*dd_12)/(W.get(1, 0)+W.get(1, 1)+W.get(1, 2)));
			double control2 = Math.sqrt((W.get(2, 0)*dd_02*dd_02+W.get(2, 1)*dd_12*dd_12)/(W.get(2, 0)+W.get(2, 1)+W.get(2, 2)));
			
			assertEquals(control0, result.get(0), 0.0001);
			assertEquals(control1, result.get(1), 0.0001);
			assertEquals(control2, result.get(2), 0.0001);
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetSimilarityMatrixRMSDTrajectory() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			Matrix result = Similarity.getSimilarityMatrixRMSD(t);
		
			assertEquals(3, result.getRowDimension());
			assertEquals(3, result.getColumnDimension());
			
			double c01 = Similarity.getRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1));
			double c02 = Similarity.getRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2));
			double c12 = Similarity.getRMSD(t.getFrameAsPointList(1), t.getFrameAsPointList(2));
			
			assertEquals(0, result.get(0, 0), 0.00001);
			assertEquals(0, result.get(1, 1), 0.00001);
			assertEquals(0, result.get(2, 2), 0.00001);
			assertEquals(c01, result.get(0, 1), 0.00001);
			assertEquals(c01, result.get(1, 0), 0.00001);
			assertEquals(c02, result.get(0, 2), 0.00001);
			assertEquals(c02, result.get(2, 0), 0.00001);
			assertEquals(c12, result.get(1, 2), 0.00001);
			assertEquals(c12, result.get(2, 1), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityMatrixRMSDTrajectoryIndexSet() {
	try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Matrix result = Similarity.getSimilarityMatrixRMSD(t, iset);
		
			assertEquals(3, result.getRowDimension());
			assertEquals(3, result.getColumnDimension());
			
			double c01 = Similarity.getRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(1), iset);
			double c02 = Similarity.getRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(2), iset);
			double c12 = Similarity.getRMSD(t.getFrameAsStructure(1), iset, t.getFrameAsStructure(2), iset);
			
			assertEquals(0, result.get(0, 0), 0.00001);
			assertEquals(0, result.get(1, 1), 0.00001);
			assertEquals(0, result.get(2, 2), 0.00001);
			assertEquals(c01, result.get(0, 1), 0.00001);
			assertEquals(c01, result.get(1, 0), 0.00001);
			assertEquals(c02, result.get(0, 2), 0.00001);
			assertEquals(c02, result.get(2, 0), 0.00001);
			assertEquals(c12, result.get(1, 2), 0.00001);
			assertEquals(c12, result.get(2, 1), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityMatrixDRMSDTrajectory() {
	try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			Matrix result = Similarity.getSimilarityMatrixDRMSD(t);
		
			assertEquals(3, result.getRowDimension());
			assertEquals(3, result.getColumnDimension());
			
			double c01 = Similarity.getDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1));
			double c02 = Similarity.getDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2));
			double c12 = Similarity.getDRMSD(t.getFrameAsPointList(1), t.getFrameAsPointList(2));
			
			assertEquals(0, result.get(0, 0), 0.00001);
			assertEquals(0, result.get(1, 1), 0.00001);
			assertEquals(0, result.get(2, 2), 0.00001);
			assertEquals(c01, result.get(0, 1), 0.00001);
			assertEquals(c01, result.get(1, 0), 0.00001);
			assertEquals(c02, result.get(0, 2), 0.00001);
			assertEquals(c02, result.get(2, 0), 0.00001);
			assertEquals(c12, result.get(1, 2), 0.00001);
			assertEquals(c12, result.get(2, 1), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityMatrixDRMSDTrajectoryIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Matrix result = Similarity.getSimilarityMatrixDRMSD(t, iset);
		
			assertEquals(3, result.getRowDimension());
			assertEquals(3, result.getColumnDimension());
			
			double c01 = Similarity.getDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(1), iset);
			double c02 = Similarity.getDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(2), iset);
			double c12 = Similarity.getDRMSD(t.getFrameAsStructure(1), iset, t.getFrameAsStructure(2), iset);
			
			assertEquals(0, result.get(0, 0), 0.00001);
			assertEquals(0, result.get(1, 1), 0.00001);
			assertEquals(0, result.get(2, 2), 0.00001);
			assertEquals(c01, result.get(0, 1), 0.00001);
			assertEquals(c01, result.get(1, 0), 0.00001);
			assertEquals(c02, result.get(0, 2), 0.00001);
			assertEquals(c02, result.get(2, 0), 0.00001);
			assertEquals(c12, result.get(1, 2), 0.00001);
			assertEquals(c12, result.get(2, 1), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityMatrixWDRMSDTrajectoryMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			t = t.getSubTrajectory(frames);
			t = t.getSubTrajectory(iset);
			
			Matrix W = Matrix.random(3, 3);
			
			Matrix result = Similarity.getSimilarityMatrixWDRMSD(t,W);
		
			assertEquals(3, result.getRowDimension());
			assertEquals(3, result.getColumnDimension());
			
			double c01 = Similarity.getWDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1),W);
			double c02 = Similarity.getWDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2),W);
			double c12 = Similarity.getWDRMSD(t.getFrameAsPointList(1), t.getFrameAsPointList(2),W);
			
			assertEquals(0, result.get(0, 0), 0.00001);
			assertEquals(0, result.get(1, 1), 0.00001);
			assertEquals(0, result.get(2, 2), 0.00001);
			assertEquals(c01, result.get(0, 1), 0.00001);
			assertEquals(c01, result.get(1, 0), 0.00001);
			assertEquals(c02, result.get(0, 2), 0.00001);
			assertEquals(c02, result.get(2, 0), 0.00001);
			assertEquals(c12, result.get(1, 2), 0.00001);
			assertEquals(c12, result.get(2, 1), 0.00001);
	
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetSimilarityMatrixWDRMSDTrajectoryIndexSetMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			t = t.getSubTrajectory(frames);
			
			Matrix W = Matrix.random(3, 3);
			
			Matrix result = Similarity.getSimilarityMatrixWDRMSD(t,iset,W);
		
			assertEquals(3, result.getRowDimension());
			assertEquals(3, result.getColumnDimension());
			
			double c01 = Similarity.getWDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(1), iset, W);
			double c02 = Similarity.getWDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(2), iset, W);
			double c12 = Similarity.getWDRMSD(t.getFrameAsStructure(1), iset, t.getFrameAsStructure(2), iset, W);
			
			assertEquals(0, result.get(0, 0), 0.00001);
			assertEquals(0, result.get(1, 1), 0.00001);
			assertEquals(0, result.get(2, 2), 0.00001);
			assertEquals(c01, result.get(0, 1), 0.00001);
			assertEquals(c01, result.get(1, 0), 0.00001);
			assertEquals(c02, result.get(0, 2), 0.00001);
			assertEquals(c02, result.get(2, 0), 0.00001);
			assertEquals(c12, result.get(1, 2), 0.00001);
			assertEquals(c12, result.get(2, 1), 0.00001);
	
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityTimeSeriesRMSDTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesRMSD(t, t.getFirstFrameAsPointList());
			
			double c01 = Similarity.getRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1));
			double c02 = Similarity.getRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2));
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetSimilarityTimeSeriesRMSDTrajectoryStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesRMSD(t, t.getFirstFrameAsStructure());
			
			double c01 = Similarity.getRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1));
			double c02 = Similarity.getRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2));
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetSimilarityTimeSeriesRMSDTrajectoryIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesRMSD(t, iset, t.getFirstFrameAsStructure(), iset);
			
			double c01 = Similarity.getRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(1), iset);
			double c02 = Similarity.getRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(2), iset);
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityTimeSeriesDRMSDTrajectoryPointList() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesDRMSD(t, t.getFirstFrameAsPointList());
			
			double c01 = Similarity.getDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1));
			double c02 = Similarity.getDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2));
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		
	}

	@Test
	public void testGetSimilarityTimeSeriesDRMSDTrajectoryStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesDRMSD(t, t.getFirstFrameAsStructure());
			
			double c01 = Similarity.getDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1));
			double c02 = Similarity.getDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2));
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityTimeSeriesDRMSDTrajectoryIndexSetStructureIndexSet() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			t = t.getSubTrajectory(frames);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesDRMSD(t, iset, t.getFirstFrameAsStructure(), iset);
			
			double c01 = Similarity.getDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(1), iset);
			double c02 = Similarity.getDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(2), iset);
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetSimilarityTimeSeriesWDRMSDTrajectoryPointListMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			t = t.getSubTrajectory(frames);
			t = t.getSubTrajectory(iset);
			
			Matrix W = Matrix.random(3, 3);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesWDRMSD(t, t.getFirstFrameAsPointList(), W);
			
			double c01 = Similarity.getWDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1), W);
			double c02 = Similarity.getWDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2), W);
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}	
	}

	@Test
	public void testGetSimilarityTimeSeriesWDRMSDTrajectoryStructureMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			t = t.getSubTrajectory(frames);
			t = t.getSubTrajectory(iset);
			
			Matrix W = Matrix.random(3, 3);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesWDRMSD(t, t.getFirstFrameAsStructure(), W);
			
			double c01 = Similarity.getWDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(1), W);
			double c02 = Similarity.getWDRMSD(t.getFrameAsPointList(0), t.getFrameAsPointList(2), W);
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}	

	}

	@Test
	public void testGetSimilarityTimeSeriesWDRMSDTrajectoryIndexSetStructureIndexSetMatrix() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");
			
			FrameIndexSet frames = new FrameIndexSet();
			frames.addFrame(4);
			frames.addFrame(9);
			frames.addFrame(18);
			
			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			t = t.getSubTrajectory(frames);
			
			Matrix W = Matrix.random(3, 3);
			
			ArrayList<Double> result = Similarity.getSimilarityTimeSeriesWDRMSD(t, iset, t.getFirstFrameAsStructure(), iset, W);
			
			double c01 = Similarity.getWDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(1), iset, W);
			double c02 = Similarity.getWDRMSD(t.getFrameAsStructure(0), iset, t.getFrameAsStructure(2), iset, W);
			
			assertEquals(0, result.get(0), 0.00001);
			assertEquals(c01, result.get(1), 0.00001);
			assertEquals(c02, result.get(2), 0.00001);
			
			
			} catch (IOException e) {
				fail(e.toString());
			}	

	}

	@Test
	public void testGetDifferenceDistanceMatrixPointListPointList() {
		
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix result = Similarity.getDifferenceDistanceMatrix(frame1, frame2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			assertEquals(0, result.get(0, 0), 0.0001);
			assertEquals(0, result.get(1, 1), 0.0001);
			assertEquals(0, result.get(2, 2), 0.0001);
			
			assertEquals(dd_01, result.get(0, 1), 0.0001);
			assertEquals(dd_01, result.get(1, 0), 0.0001);
			
			assertEquals(dd_02, result.get(0, 2), 0.0001);
			assertEquals(dd_02, result.get(2, 0), 0.0001);

			assertEquals(dd_12, result.get(1, 2), 0.0001);
			assertEquals(dd_12, result.get(2, 1), 0.0001);

			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDifferenceDistanceMatrixStructureStructure() {
		try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix result = Similarity.getDifferenceDistanceMatrix(frameS1, frameS2);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			assertEquals(0, result.get(0, 0), 0.0001);
			assertEquals(0, result.get(1, 1), 0.0001);
			assertEquals(0, result.get(2, 2), 0.0001);
			
			assertEquals(dd_01, result.get(0, 1), 0.0001);
			assertEquals(dd_01, result.get(1, 0), 0.0001);
			
			assertEquals(dd_02, result.get(0, 2), 0.0001);
			assertEquals(dd_02, result.get(2, 0), 0.0001);

			assertEquals(dd_12, result.get(1, 2), 0.0001);
			assertEquals(dd_12, result.get(2, 1), 0.0001);

			
			} catch (IOException e) {
				fail(e.toString());
			}		

	}

	@Test
	public void testGetDifferenceDistanceMatrixStructureIndexSetStructureIndexSet() {
try {
			
			Structure s = IOData.readStructureFromGRO("dataset//example_calpha.gro");
			Trajectory t = IOData.readDumpedTrajectory(s, "dataset//example_calpha.dxtc");

			IndexSet iset = new IndexSet();
			iset.addIndex(10);
			iset.addIndex(20);
			iset.addIndex(30);
			
			Structure frameS1 = t.getFrameAsStructure(9);
			Structure frameS2 = t.getFrameAsStructure(17);

			Structure s1 = (Structure)frameS1.clone();
			Structure s2 = (Structure)frameS2.clone();
			
			frameS1 = frameS1.getSubStructure(iset);
			frameS2 = frameS2.getSubStructure(iset);
			
			PointList frame1 = frameS1.getAllAtomCoordinates();
			PointList frame2 = frameS2.getAllAtomCoordinates();
			
			Matrix result = Similarity.getDifferenceDistanceMatrix(s1, iset , s2, iset);
			
			double d1_01 = frame1.getPoint(0).distance(frame1.getPoint(1));
			double d1_02 = frame1.getPoint(0).distance(frame1.getPoint(2));
			double d1_12 = frame1.getPoint(1).distance(frame1.getPoint(2));
			
			double d2_01 = frame2.getPoint(0).distance(frame2.getPoint(1));
			double d2_02 = frame2.getPoint(0).distance(frame2.getPoint(2));
			double d2_12 = frame2.getPoint(1).distance(frame2.getPoint(2));
			
			double dd_01 = d1_01-d2_01;
			double dd_02 = d1_02-d2_02;
			double dd_12 = d1_12-d2_12;
			
			assertEquals(0, result.get(0, 0), 0.0001);
			assertEquals(0, result.get(1, 1), 0.0001);
			assertEquals(0, result.get(2, 2), 0.0001);
			
			assertEquals(dd_01, result.get(0, 1), 0.0001);
			assertEquals(dd_01, result.get(1, 0), 0.0001);
			
			assertEquals(dd_02, result.get(0, 2), 0.0001);
			assertEquals(dd_02, result.get(2, 0), 0.0001);

			assertEquals(dd_12, result.get(1, 2), 0.0001);
			assertEquals(dd_12, result.get(2, 1), 0.0001);

			
			} catch (IOException e) {
				fail(e.toString());
			}		


	}

	@Test
	public void testFindSimilarFramesRMSD() {
		Trajectory t = new Trajectory();
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(21,21,21));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2.1,2.1,2.1));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(1.9,1.9,1.9));
	
		PointList frame5 = new PointList();
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(-10,-10,-10));
		
		PointList frame6 = new PointList();
		frame6.addPoint(new Point3D(2,2,2));
		frame6.addPoint(new Point3D(2,2,2));
		frame6.addPoint(new Point3D(2.05,2.05,2.05));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		t.addFrame(frame5);
		t.addFrame(frame6);
		
		FrameIndexSet result = Similarity.findSimilarFramesRMSD(t, frame1, 0.1);
		result.setName("");
		
		FrameIndexSet control = new FrameIndexSet("");
		control.addFrame(0);
		control.addFrame(2);
		control.addFrame(3);
		control.addFrame(5);

		assertEquals(control, result);
	}

	@Test
	public void testFindSimilarFramesDRMSDTrajectoryPointListDouble() {
		Trajectory t = new Trajectory();
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(21,21,21));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2.1,2.1,2.1));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(1.9,1.9,1.9));
	
		PointList frame5 = new PointList();
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(-10,-10,-10));
		
		PointList frame6 = new PointList();
		frame6.addPoint(new Point3D(2,2,2));
		frame6.addPoint(new Point3D(2,2,2));
		frame6.addPoint(new Point3D(2.05,2.05,2.05));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		t.addFrame(frame5);
		t.addFrame(frame6);
		
		FrameIndexSet result = Similarity.findSimilarFramesDRMSD(t, frame1, 0.5);
		result.setName("");
		
		FrameIndexSet control = new FrameIndexSet("");
		control.addFrame(0);
		control.addFrame(2);
		control.addFrame(3);
		control.addFrame(5);

		assertEquals(control, result);

	}

	@Test
	public void testFindSimilarFramesDRMSDTrajectoryMatrixDouble() {
		Trajectory t = new Trajectory();
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(21,21,21));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2.1,2.1,2.1));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(1.9,1.9,1.9));
	
		PointList frame5 = new PointList();
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(-10,-10,-10));
		
		PointList frame6 = new PointList();
		frame6.addPoint(new Point3D(2,2,2));
		frame6.addPoint(new Point3D(2,2,2));
		frame6.addPoint(new Point3D(2.05,2.05,2.05));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		t.addFrame(frame5);
		t.addFrame(frame6);
		
		Matrix M = Distances.getDistanceMatrix(frame1);
		FrameIndexSet result = Similarity.findSimilarFramesDRMSD(t, M, 0.5);
		result.setName("");
		
		FrameIndexSet control = new FrameIndexSet("");
		control.addFrame(0);
		control.addFrame(2);
		control.addFrame(3);
		control.addFrame(5);

		assertEquals(control, result);

	}

	@Test
	public void testGetMedoidRMSD() {
		Trajectory t = new Trajectory();
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(21,21,21));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(2,2,2));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2.1,2.1,2.1));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(1.9,1.9,1.9));
	
		PointList frame5 = new PointList();
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(2,2,2));
		frame5.addPoint(new Point3D(-10,-10,-10));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		t.addFrame(frame5);
		
		PointList result = Similarity.getMedoidRMSD(t);
		
		assertEquals(frame2, result);
	}

	@Test
	public void testGetMedoidDRMSD() {
		Trajectory t = new Trajectory();
		
		PointList frame1 = new PointList();
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(2,2,2));
		frame1.addPoint(new Point3D(21,21,21));
		
		PointList frame2 = new PointList();
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(2,2,2));
		frame2.addPoint(new Point3D(2,2,2));
		
		PointList frame3 = new PointList();
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2,2,2));
		frame3.addPoint(new Point3D(2.1,2.1,2.1));
		
		PointList frame4 = new PointList();
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(2,2,2));
		frame4.addPoint(new Point3D(-10,-10,-10));
		
		t.addFrame(frame1);
		t.addFrame(frame2);
		t.addFrame(frame3);
		t.addFrame(frame4);
		
		PointList result = Similarity.getMedoidDRMSD(t);
		
		assertEquals(frame3, result);

	}

}
