/**
 * 
 * Written by Mrton Mnz and Philip C Biggin
 * Copyright (c) University of Oxford, United Kingdom
 * Visit http://sbcb.bioch.ox.ac.uk/jgromacs/
 * 
 * This source code file is part of JGromacs v1.0.
 * 
 * JGromacs v1.0 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * JGromacs v1.0. is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with JGromacs v1.0. If not, see <http://www.gnu.org/licenses/>.
 * 
 */


import java.util.ArrayList;

import jama.Matrix;
import jgromacs.analysis.Superposition;
import jgromacs.data.Atom;
import jgromacs.data.IndexSet;
import jgromacs.data.Point3D;
import jgromacs.data.PointList;
import jgromacs.data.Residue;
import jgromacs.data.Structure;
import jgromacs.data.Trajectory;
import junit.framework.TestCase;

import org.junit.Test;

public class SuperpositionTest extends TestCase{

	@Test
	public void testSuperposeToPointListPointList() {
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		
		pl2.translate(new Point3D(1,-3,2.4));
		
		pl2 = Superposition.superposeTo(pl2, pl1);
		
		assertEquals(pl1, pl2);
		
	}

	@Test
	public void testSuperposeToStructureStructure() {

		Structure s1 = new Structure();
		Residue r1 = new Residue();
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		s1.addResidue(r1);
		
		Structure s2 = new Structure();
		Residue r2 = new Residue();
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
		s2.addResidue(r2);
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		
		pl2.translate(new Point3D(1,-3,2.4));
		
		s1.setAllAtomCoordinates(pl1);
		s2.setAllAtomCoordinates(pl2);
		
		s2 = Superposition.superposeTo(s2, s1);
		
		assertEquals(s1.getAllAtomCoordinates(), s2.getAllAtomCoordinates());
		
	}

	@Test
	public void testSuperposeToStructureIndexSetStructureIndexSet() {

		Structure s1 = new Structure();
		Residue r1 = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r1.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r1.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r1.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		r1.addAtom(a4);
		Atom a5 = new Atom();
		a5.setIndex(5);
		r1.addAtom(a5);
		s1.addResidue(r1);
		
		Structure s2 = new Structure();
		Residue r2 = new Residue();
		Atom a6 = new Atom();
		a6.setIndex(1);
		r2.addAtom(a6);
		Atom a7 = new Atom();
		a7.setIndex(2);
		r2.addAtom(a7);
		Atom a8 = new Atom();
		a8.setIndex(3);
		r2.addAtom(a8);
		Atom a9 = new Atom();
		a9.setIndex(4);
		r2.addAtom(a9);
		Atom a10 = new Atom();
		a10.setIndex(5);
		r2.addAtom(a10);
		s2.addResidue(r2);
		
		
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		
		pl2.translate(new Point3D(1,-3,2.4));
		
		// Additional points:
		
		pl1.addPoint(new Point3D(3.2,-1.4,-4.3));
		pl1.addPoint(new Point3D(2.9,11.3,6.6));
		
		pl2.addPoint(new Point3D(9.2,1.1,-4.1));
		pl2.addPoint(new Point3D(12.2,-3.1,6.2));
		
		s1.setAllAtomCoordinates(pl1);
		s2.setAllAtomCoordinates(pl2);
		
		IndexSet iset = new IndexSet();
		iset.addIndex(1);
		iset.addIndex(2);
		iset.addIndex(3);
		
		s2 = Superposition.superposeTo(s2, iset, s1, iset);
		
		assertEquals(s1.getAtomByIndex(1).getCoordinates(), s2.getAtomByIndex(1).getCoordinates());
		assertEquals(s1.getAtomByIndex(2).getCoordinates(), s2.getAtomByIndex(2).getCoordinates());
		assertEquals(s1.getAtomByIndex(3).getCoordinates(), s2.getAtomByIndex(3).getCoordinates());
		
	}

	@Test
	public void testSuperposeToTrajectoryPointList() {
		Structure s = new Structure();
		Residue r1 = new Residue();
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		s.addResidue(r1);
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		PointList pl3 = (PointList)pl1.clone();
		PointList pl4 = (PointList)pl1.clone();
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		t.addFrame(pl1);
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		pl2.translate(new Point3D(1,-3,2.4));
		t.addFrame(pl2);
		
		pl3.rotate(rot); 
		pl3.translate(new Point3D(1.9,2.1,-4.2));
		t.addFrame(pl3);
		
		pl4.translate(new Point3D(9,2,3));
		t.addFrame(pl4);
		
		Trajectory t2 = Superposition.superposeTo(t, pl1);
		
		assertEquals(pl1, t2.getFrameAsPointList(0));
		assertEquals(pl1, t2.getFrameAsPointList(1));
		assertEquals(pl1, t2.getFrameAsPointList(2));
		assertEquals(pl1, t2.getFrameAsPointList(3));
	}

	@Test
	public void testSuperposeToTrajectoryStructureIndexSet() {
		Structure s = new Structure();
		Residue r = new Residue();
		Atom a1 = new Atom();
		a1.setIndex(1);
		r.addAtom(a1);
		Atom a2 = new Atom();
		a2.setIndex(2);
		r.addAtom(a2);
		Atom a3 = new Atom();
		a3.setIndex(3);
		r.addAtom(a3);
		Atom a4 = new Atom();
		a4.setIndex(4);
		r.addAtom(a4);
		s.addResidue(r);
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		PointList pl3 = (PointList)pl1.clone();
		PointList pl4 = (PointList)pl1.clone();
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
		pl1.addPoint(new Point3D(9,9,-9)); // addiitonal point
		t.addFrame(pl1);
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		pl2.translate(new Point3D(1,-3,2.4));
		pl2.addPoint(new Point3D(-2,2,2)); // Additional point
		t.addFrame(pl2);
		
		pl3.rotate(rot); 
		pl3.translate(new Point3D(1.9,2.1,-4.2));
		pl3.addPoint(new Point3D(3,-2,3)); // Additional point
		t.addFrame(pl3);
		
		pl4.translate(new Point3D(9,2,3));
		pl4.addPoint(new Point3D(1,1,-1)); // Additional point
		t.addFrame(pl4);
		
		IndexSet iset = new IndexSet();
		iset.addIndex(1);
		iset.addIndex(2);
		iset.addIndex(3);
		
		Trajectory t2 = Superposition.superposeTo(t, s, iset);
		
		assertEquals(t2.getFrameAsPointList(0).getPoint(0), t2.getFrameAsPointList(1).getPoint(0));
		assertEquals(t2.getFrameAsPointList(0).getPoint(1), t2.getFrameAsPointList(1).getPoint(1));
		assertEquals(t2.getFrameAsPointList(0).getPoint(2), t2.getFrameAsPointList(1).getPoint(2));
	
		assertEquals(t2.getFrameAsPointList(0).getPoint(0), t2.getFrameAsPointList(2).getPoint(0));
		assertEquals(t2.getFrameAsPointList(0).getPoint(1), t2.getFrameAsPointList(2).getPoint(1));
		assertEquals(t2.getFrameAsPointList(0).getPoint(2), t2.getFrameAsPointList(2).getPoint(2));
		
		assertEquals(t2.getFrameAsPointList(0).getPoint(0), t2.getFrameAsPointList(3).getPoint(0));
		assertEquals(t2.getFrameAsPointList(0).getPoint(1), t2.getFrameAsPointList(3).getPoint(1));
		assertEquals(t2.getFrameAsPointList(0).getPoint(2), t2.getFrameAsPointList(3).getPoint(2));
	
	}

	@Test
	public void testWeightedSuperposeToPointListPointListArrayListOfDouble() {
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		
		pl2.translate(new Point3D(1,-3,2.4));
		
		// Additional points
		pl1.addPoint(new Point3D(1,2,3));
		pl1.addPoint(new Point3D(3,2,1));
		pl2.addPoint(new Point3D(-1,2,-3));
		pl2.addPoint(new Point3D(3,-2,1));
		
		ArrayList<Double> weights = new ArrayList<Double>();
		weights.add(1.0);
		weights.add(1.0);
		weights.add(1.0);
		weights.add(0.0);
		weights.add(0.0);
		
		pl2 = Superposition.weightedSuperposeTo(pl2, pl1, weights);
		
		assertEquals(pl1.getPoint(0), pl2.getPoint(0));
		assertEquals(pl1.getPoint(1), pl2.getPoint(1));
		assertEquals(pl1.getPoint(2), pl2.getPoint(2));
		
	}

	@Test
	public void testWeightedSuperposeToStructureStructureArrayListOfDouble() {

		Structure s1 = new Structure();
		Residue r1 = new Residue();
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		s1.addResidue(r1);
		
		Structure s2 = new Structure();
		Residue r2 = new Residue();
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
		r2.addAtom(new Atom());
		s2.addResidue(r2);
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		
		pl2.translate(new Point3D(1,-3,2.4));
		
		// Additional points
		pl1.addPoint(new Point3D(1,2,3));
		pl1.addPoint(new Point3D(3,2,1));
		pl2.addPoint(new Point3D(-1,2,-3));
		pl2.addPoint(new Point3D(3,-2,1));
		
		s1.setAllAtomCoordinates(pl1);
		s2.setAllAtomCoordinates(pl2);
		
		ArrayList<Double> weights = new ArrayList<Double>();
		weights.add(1.0);
		weights.add(1.0);
		weights.add(1.0);
		weights.add(0.0);
		weights.add(0.0);
		
		s2 = Superposition.weightedSuperposeTo(s2, s1, weights);
		
		assertEquals(s1.getAtom(0).getCoordinates(), s2.getAtom(0).getCoordinates());
		assertEquals(s1.getAtom(1).getCoordinates(), s2.getAtom(1).getCoordinates());
		assertEquals(s1.getAtom(2).getCoordinates(), s2.getAtom(2).getCoordinates());
		
	}

	@Test
	public void testWeightedSuperposeToTrajectoryPointListArrayListOfDouble() {

		Structure s = new Structure();
		Residue r1 = new Residue();
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		s.addResidue(r1);
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		PointList pl3 = (PointList)pl1.clone();
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
	
		// adding new point
		pl1.addPoint(new Point3D(-1,-1,-1));
		t.addFrame(pl1);
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		pl2.translate(new Point3D(1,-3,2.4));
		pl2.addPoint(new Point3D(4,-1,-2)); // adding new point
		t.addFrame(pl2);
		
		pl3.rotate(rot); 
		pl3.translate(new Point3D(1.9,2.1,-4.2));
		pl3.addPoint(new Point3D(9,0,9)); // adding new point
		t.addFrame(pl3);
		
		ArrayList<Double> weights = new ArrayList<Double>();
		weights.add(1.0);
		weights.add(1.0);
		weights.add(1.0);
		weights.add(0.0);
		
		Trajectory t2 = Superposition.weightedSuperposeTo(t, pl1, weights);
		
		assertEquals(pl1.getPoint(0), t2.getFrameAsPointList(1).getPoint(0));
		assertEquals(pl1.getPoint(1), t2.getFrameAsPointList(1).getPoint(1));
		assertEquals(pl1.getPoint(2), t2.getFrameAsPointList(1).getPoint(2));
		
		assertEquals(pl1.getPoint(0), t2.getFrameAsPointList(2).getPoint(0));
		assertEquals(pl1.getPoint(1), t2.getFrameAsPointList(2).getPoint(1));
		assertEquals(pl1.getPoint(2), t2.getFrameAsPointList(2).getPoint(2));
		
	}

	@Test
	public void testWeightedSuperposeToTrajectoryStructureArrayListOfDouble() {

		Structure s = new Structure();
		Residue r1 = new Residue();
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		r1.addAtom(new Atom());
		s.addResidue(r1);
		
		PointList pl1 = new PointList();
		pl1.addPoint(new Point3D(1,1,1));
		pl1.addPoint(new Point3D(2,2,2));
		pl1.addPoint(new Point3D(3,3,3));
		
		PointList pl2 = (PointList)pl1.clone();
		PointList pl3 = (PointList)pl1.clone();
	
		Trajectory t = new Trajectory();
		t.setStructure(s);
	
		// adding new point
		pl1.addPoint(new Point3D(-1,-1,-1));
		t.addFrame(pl1);
		
		Matrix rot = new Matrix(3,3,0);
		rot.set(0, 0, 0);
		rot.set(0, 1, 0);
		rot.set(0, 2, 1);
		rot.set(1, 0, 1);
		rot.set(1, 1, 0);
		rot.set(1, 2, 0);
		rot.set(2, 0, 0);
		rot.set(2, 1, 1);
		rot.set(2, 2, 0);
		
		pl2.rotate(rot); 
		pl2.translate(new Point3D(1,-3,2.4));
		pl2.addPoint(new Point3D(4,-1,-2)); // adding new point
		t.addFrame(pl2);
		
		pl3.rotate(rot); 
		pl3.translate(new Point3D(1.9,2.1,-4.2));
		pl3.addPoint(new Point3D(9,0,9)); // adding new point
		t.addFrame(pl3);
		
		ArrayList<Double> weights = new ArrayList<Double>();
		weights.add(1.0);
		weights.add(1.0);
		weights.add(1.0);
		weights.add(0.0);
		
		s.setAllAtomCoordinates(pl1);
		
		Trajectory t2 = Superposition.weightedSuperposeTo(t, s, weights);
		
		assertEquals(pl1.getPoint(0), t2.getFrameAsPointList(1).getPoint(0));
		assertEquals(pl1.getPoint(1), t2.getFrameAsPointList(1).getPoint(1));
		assertEquals(pl1.getPoint(2), t2.getFrameAsPointList(1).getPoint(2));
		
		assertEquals(pl1.getPoint(0), t2.getFrameAsPointList(2).getPoint(0));
		assertEquals(pl1.getPoint(1), t2.getFrameAsPointList(2).getPoint(1));
		assertEquals(pl1.getPoint(2), t2.getFrameAsPointList(2).getPoint(2));
	
	}

}
